/*
 * Copyright (C) 2010 hidetake.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hidetake.jsonizer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import net.arnx.jsonic.JSON;

import org.hidetake.jsonizer.JsonWriter;
import org.hidetake.jsonizer.PropertyGetter;
import org.junit.Before;
import org.junit.Test;


public class JsonWriterPerfTest
{

	public static class Hoge
	{
		private int count;
		private String message;
		public int getCount()
		{
			return count;
		}
		public void setCount(int count)
		{
			this.count = count;
		}
		public String getMessage()
		{
			return message;
		}
		public void setMessage(String message)
		{
			this.message = message;
		}
	}

	public static class HogeGetter extends PropertyGetter<Hoge>
	{
		@Override
		public void get(Hoge m)
		{
//			property("count", m.getCount());
//			property("msg", m.getMessage());
			property(m.getCount()).as("count");
			property(m.getMessage()).as("message");
		}
	}

	private static final Logger logger = Logger.getLogger(JsonWriterPerfTest.class.getName());
	private StringBuilder result;
	private long time;

	@Before
	public void setUp()
	{
		result = new StringBuilder();
	}

	@Test
	public void warmUp() throws Exception
	{
		logger.info(jsonic(5000));
		logger.info(inner_getter(5000));
		logger.info(static_getter(5000));
		logger.info("done");
	}
	
	@Test
	public void _jsonic_10000() throws Exception
	{
		logger.info(jsonic(10000));
		logger.info(jsonic(10000));
		logger.info(jsonic(10000));
	}

	@Test
	public void _inner_getter_10000() throws Exception
	{
		logger.info(inner_getter(10000));
		logger.info(inner_getter(10000));
		logger.info(inner_getter(10000));
	}

	@Test
	public void _jsonic_100000() throws Exception
	{
		logger.info(jsonic(100000));
		logger.info(jsonic(100000));
		logger.info(jsonic(100000));
	}

	@Test
	public void _inner_getter_100000() throws Exception
	{
		logger.info(inner_getter(100000));
		logger.info(inner_getter(100000));
		logger.info(inner_getter(100000));
	}

	private String jsonic(int count) throws Exception
	{
		List<?> list = makeList(count);
		start();
		JSON.encode(list, result);
		return count+" items: "+endMs();
	}

	private String static_getter(int count) throws Exception
	{
		List<?> list = makeList(count);
		start();
		Map<Class<?>, PropertyGetter<?>> map = new HashMap<Class<?>, PropertyGetter<?>>();
		map.put(Hoge.class, new HogeGetter());
		JsonWriter jsonWriter = new JsonWriter(result, map);
		jsonWriter.write(list);
		return count+" items: "+endMs();
	}

	private String inner_getter(int count) throws Exception
	{
		List<?> list = makeList(count);
		start();
		Map<Class<?>, PropertyGetter<?>> map = new HashMap<Class<?>, PropertyGetter<?>>();
		map.put(Hoge.class, new PropertyGetter<Hoge>()
		{
			public void get(Hoge m) {
				property("count", m.getCount());
				property("message", m.getMessage());
			};
		});
		JsonWriter jsonWriter = new JsonWriter(result, map);
		jsonWriter.write(list);
		return count+" items: "+endMs();
	}

	private List<?> makeList(int size)
	{
		List<Object> list = new ArrayList<Object>();
		for(int i = 0; i < size; i++) {
			Hoge hoge = new Hoge();
			hoge.setCount(i);
			hoge.setMessage("OK");
			list.add(hoge);
			list.add(i);
		}
		return list;
	}

	private void start()
	{
		time = System.currentTimeMillis();
	}

	private long end()
	{
		return System.currentTimeMillis() - time;
	}

	private String endMs()
	{
		return end() + " ms";
	}

}
